@@ -86,7 +86,7 @@ file_validate (void *cls,
 
   (void) cls;
   if (NULL == data)
-    return GNUNET_NO;
+    return GNUNET_SYSERR;
   filename = GNUNET_STRINGS_data_to_string_alloc (data,
                                                   data_length);
   flag = false;
@@ -100,7 +100,7 @@ file_validate (void *cls,
     }
   }
   if (flag)
-    return GNUNET_NO;
+    return GNUNET_SYSERR;
   GNUNET_free (filename);
   return GNUNET_OK;
 }
