@@ -681,14 +681,16 @@ AH_handler_policy_post (
     {
       const char *lens;
       unsigned long len;
+      char dummy;
 
       lens = MHD_lookup_connection_value (connection,
                                           MHD_HEADER_KIND,
                                           MHD_HTTP_HEADER_CONTENT_LENGTH);
       if ( (NULL == lens) ||
            (1 != sscanf (lens,
-                         "%lu",
-                         &len)) )
+                         "%lu%c",
+                         &len,
+                         &dummy)) )
       {
         GNUNET_break_op (0);
         return TALER_MHD_reply_with_error (
@@ -794,10 +796,12 @@ AH_handler_policy_post (
       if (NULL != long_poll_timeout_ms)
       {
         unsigned int timeout;
+        char dummy;
 
         if (1 != sscanf (long_poll_timeout_ms,
-                         "%u",
-                         &timeout))
+                         "%u%c",
+                         &timeout,
+                         &dummy))
         {
           GNUNET_break_op (0);
           return TALER_MHD_reply_with_error (connection,
