@@ -71,10 +71,16 @@ struct ANASTASIS_ChallengeDetails
   const char *instructions;
 
   /**
-   * true if challenged was already solved, else false.
+   * true if challenge was already solved, else false.
    */
   bool solved;
 
+  /**
+   * true if challenge is awaiting asynchronous
+   * resolution by the user.
+   */
+  bool async;
+
 };
 
 
@@ -131,7 +137,21 @@ enum ANASTASIS_ChallengeStatus
   /**
    * The rate limit for solving the challenge was exceeded.
    */
-  ANASTASIS_CHALLENGE_STATUS_RATE_LIMIT_EXCEEDED
+  ANASTASIS_CHALLENGE_STATUS_RATE_LIMIT_EXCEEDED,
+
+  /**
+   * The user did not satisfy the (external) authentication
+   * challenge in time. The request should be repeated
+   * later and may then succeed.
+   */
+  ANASTASIS_CHALLENGE_STATUS_AUTH_TIMEOUT,
+
+  /**
+   * Plugin-specific ("external") instructions for how to solve the
+   * challenge are provided.
+   */
+  ANASTASIS_CHALLENGE_STATUS_EXTERNAL_INSTRUCTIONS
+
 
 };
 
@@ -159,7 +179,7 @@ struct ANASTASIS_ChallengeStartResponse
 
     /**
      * Challenge details provided if
-     * @e cs is #ANASTASIS_CHALLENGE_STATUS_INSTRUCTIONS.
+     * @e cs is #ANASTASIS_CHALLENGE_STATUS_INSTRUCTIONS
      */
     struct
     {
@@ -188,6 +208,12 @@ struct ANASTASIS_ChallengeStartResponse
     } open_challenge;
 
 
+    /**
+     * Response with details if
+     * @e cs is #ANASTASIS_CHALLENGE_STATUS_EXTERNAL_INSTRUCTIONS.
+     */
+    const json_t *external_challenge;
+
     /**
      * Response with URL to redirect the user to, if
      * @e cs is #ANASTASIS_CHALLENGE_STATUS_REDIRECT_FOR_AUTHENTICATION.
