@@ -126,6 +126,14 @@ struct ANASTASIS_AuthorizationPlugin
    */
   bool payment_plugin_managed;
 
+  /**
+   * The plugin expects the "code" in the "start" function to be
+   * provided by the user and not generated by the Anastasis
+   * backend. The plugin will then validate the code using its own
+   * means.  Used by TOTP.
+   */
+  bool user_provided_code;
+
   /**
    * How often are retries allowed for challenges created
    * by this plugin?
