@@ -232,9 +232,12 @@ struct ANASTASIS_DatabasePlugin
   * Does not return anything, as we will continue regardless of the outcome.
   *
   * @param cls the `struct PostgresClosure` with the plugin-specific state
+  * @return #GNUNET_OK if everything is fine
+  *         #GNUNET_NO if a transaction was rolled back
+  *         #GNUNET_SYSERR on hard errors
   */
-  void
-  (*preflight) (void *cls);
+  enum GNUNET_GenericReturnValue
+  (*preflight)(void *cls);
 
   /**
   * Check that the database connection is still up.
@@ -630,7 +633,7 @@ struct ANASTASIS_DatabasePlugin
   (*mark_challenge_code_satisfied)(
     void *cls,
     const struct ANASTASIS_CRYPTO_TruthUUIDP *truth_uuid,
-    const uint64_t code);
+    uint64_t code);
 
 
   /**
