@@ -466,9 +466,22 @@ enum ANASTASIS_KeyShareDownloadStatus
 
   /**
    * Too many attempts to solve the challenge were made in a short
-   * time. Try again laster.
+   * time. Try again later.
    */
-  ANASTASIS_KSD_RATE_LIMIT_EXCEEDED
+  ANASTASIS_KSD_RATE_LIMIT_EXCEEDED,
+
+  /**
+   * The user did not satisfy the (external)
+   * authentication check until the request timeout
+   * was reached. The client should try again later.
+   */
+  ANASTASIS_KSD_AUTHENTICATION_TIMEOUT,
+
+  /**
+   * The plugin provided external challenge instructions
+   * that should be followed. They are method-specific.
+   */
+  ANASTASIS_KSD_EXTERNAL_CHALLENGE_INSTRUCTIONS
 
 };
 
@@ -577,6 +590,12 @@ struct ANASTASIS_KeyShareDownloadDetails
 
     } server_failure;
 
+    /**
+     * External challenge instructions, if @e status is
+     * #ANASTASIS_KSD_EXTERNAL_CHALLENGE_INSTRUCTIONS.
+     */
+    const json_t *external_challenge;
+
   } details;
 };
 
