@@ -1684,10 +1684,13 @@ select_challenge_cb (void *cls,
       json_object_set_new (sctx->state,
                            "selected_challenge_uuid",
                            GNUNET_JSON_from_data_auto (&cd->uuid)));
-    if (0 == strcmp ("question",
-                     cd->type))
+    if ( (0 == strcmp ("question",
+                       cd->type)) ||
+         (0 == strcmp ("totp",
+                       cd->type)) )
     {
-      /* security question, immediately request user to answer it */
+      /* security question or TOTP:
+         immediately request user to answer it */
       set_state (sctx->state,
                  ANASTASIS_RECOVERY_STATE_CHALLENGE_SOLVING);
       sctx->cb (sctx->cb_cls,
@@ -2864,6 +2867,23 @@ ANASTASIS_REDUX_recovery_challenge_begin_ (json_t *state,
     const char *url;
 
     json_object_foreach (providers, url, prov) {
+      bool disabled = false;
+      struct GNUNET_JSON_Specification spec[] = {
+        GNUNET_JSON_spec_mark_optional (
+          GNUNET_JSON_spec_bool ("disabled",
+                                 &disabled)),
+        GNUNET_JSON_spec_end ()
+      };
+
+      if (GNUNET_OK !=
+          GNUNET_JSON_parse (prov,
+                             spec,
+                             NULL, NULL))
+      {
+        /* skip malformed provider entry */
+        GNUNET_break_op (0);
+        continue;
+      }
       begin_query_provider (rss,
                             url);
     }
